@@ -1,5 +1,5 @@
 /* helpmenu.h  -  The GNU Privacy Assistant
- *      Copyright (C) 2000 Free Software Foundation, Inc.
+ *	Copyright (C) 2000 Free Software Foundation, Inc.
  *
  * This file is part of GPA
  *
@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
-extern void help_version (void);
+extern void help_about (void);
 extern void help_license (void);
 extern void help_warranty (void);
 extern void help_help (void);
